from model.model_statistics import *
from mesa import Agent
import random
from agents.initialization import BeliefsInitializer

def lower_price(lowest, valuation, agent_id):
    if(valuation<lowest.price): 
        if(lowest.agent.unique_id == agent_id):
            return lowest.price
        else:
           return lowest.price-0.01
    return 1

def increase_price(highest, valuation, agent_id):
    if(valuation>highest.price): 
        if(highest.agent.unique_id == agent_id):
            return highest.price
        else:
           return highest.price + 0.01
    return 0

class ClearingPriceAgent(Agent):
    """ An agent who's ask_price is slightly lower than the price of the lowest ask in the queue,
     and whose bid is slightly higher than the highest bid in the queue."""
    def __init__(agent, unique_id, wealth, model):
        super().__init__(unique_id, model)
        agent.wealth = wealth
        agent.belief = BeliefsInitializer().init_belief(model)
        agent.contracts_alpha = 0
        agent.contracts_beta = 0

    # Determines the ask price for alpha contracts
    def ask_price_alpha(self, model):
        if(len(model.ask_queue_alpha)>0):
            return lower_price(get_lowest_ask(model.ask_queue_alpha), self.belief, self.unique_id)
        else:
            return 1

    # Determines the ask price for beta contracts
    def ask_price_beta(self, model):
        if(len(model.ask_queue_beta)>0):
            return lower_price(get_lowest_ask(model.ask_queue_beta), 1-self.belief, self.unique_id)
        else:
            return 1

    # Determines the bid price for alpha contracts
    def bid_price_alpha(self, model):
        if(len(model.bid_queue_alpha)>0):
            high = get_highest_bid(model.bid_queue_alpha)
            price = increase_price(high, self.belief, self.unique_id)
        else:
            price = 0.01
        if(self.wealth >= price):
            return price
        else:
            return 0

    # Determines the bid price for beta contracts
    def bid_price_beta(self, model):
        if(len(model.bid_queue_beta)>0):
            high = get_highest_bid(model.bid_queue_beta)
            price = increase_price(high, 1-self.belief, self.unique_id)
        else:
            price = 0.01
        if(self.wealth >= price):
            return price
        else:
            return 0

