from agents.initialization import BeliefsInitializer
from mesa import Agent
import random

class FixedMarkupAgent(Agent):
    """ An agent with a fixed markup added to its belief."""
    def __init__(agent, unique_id, wealth, model):
        super().__init__(unique_id, model)
        agent.wealth = wealth
        agent.belief = BeliefsInitializer().init_belief(model)
        agent.contracts_alpha = 0
        agent.contracts_beta = 0
        agent.markup = 0.05

    # Determines the ask price for alpha contracts
    def ask_price_alpha(self, model):
        return self.belief + self.markup
    
    # Determines the ask price for beta contracts
    def ask_price_beta(self, model):
        return 1 - self.belief + self.markup
    
    # Determines the bid price for alpha contracts
    def bid_price_alpha(self, model):
        return self.belief - self.markup

    # Determines the bid price for beta contracts
    def bid_price_beta(self, model):
        return 1 - self.belief - self.markup