import random
from scipy.stats import truncnorm

# A function to get a single belief value from a truncated normal distribution
def get_truncated_normal(mean, sd, low, upp):
    return truncnorm((low-mean)/sd, (upp-mean)/sd, loc=mean, scale=sd)

# Initialization of beliefs
class BeliefsInitializer():
    def init_belief(self, model):
        if(model.belief_distribution == "Random"):
            return (random.randint(0,100)/100)
        elif(model.belief_distribution == "Equal"):
            return 0.5
        elif(model.belief_distribution == "Normal"):
            x = get_truncated_normal(model.mean_belief, model.sigma_belief, 0, 1)
            return x.rvs()

# Initialization of wealth
class WealthInitializer():
    def init_wealth(self,model):
        if(model.wealth_distribution == "Equal"):
            return model.total_wealth/model.num_agents
        elif(model.wealth_distribution == "Random"):
            return random.randint(0,100)/100
        elif(model.wealth_distribution == "Normal"):
            wealth = np.random.normal(model.mean_wealth,model.sigma_wealth)
            while(wealth < 0):
                wealth = np.random.normal(model.mean_wealth,model.sigma_wealth)
            return wealth
        elif(model.wealth_distribution == "Pareto"):
            return random.paretovariate(model.alpha_shape)
        return

    # After the initial initialization adapts the wealths so they add up to the specified total_wealth
    def adapt_wealth(self,model):
        sum_wealth = sum([a.wealth for a in model.schedule.agents])
        for a in model.schedule.agents:
            a.wealth = (a.wealth/sum_wealth) * model.total_wealth

from agents.strategic import StrategicAgent
from agents.clearing_price import ClearingPriceAgent
from agents.fixed_markup import FixedMarkupAgent
from agents.random_markup import RandomMarkupAgent
# Initializes the type of agent, 
class TypeInitializer():
    def init_type_agent(self, i, wealth, model):
        if(i<model.num_adaptable):
            return ClearingPriceAgent(i, wealth, model)
        elif(i<model.num_adaptable+model.num_fixed_markup):
            return FixedMarkupAgent(i, wealth, model)
        elif(i<model.num_adaptable+model.num_fixed_markup+model.num_random_markup):
            return RandomMarkupAgent(i, wealth, model)
        elif(i<model.num_adaptable+model.num_fixed_markup+model.num_random_markup+model.num_strategic):
            return StrategicAgent(i, wealth, model)
        return

# Initialization of agents
def init_agents(model):
    wealthinit = WealthInitializer()
    typeinit = TypeInitializer()
    for i in range(model.num_agents):
        model.schedule.add(typeinit.init_type_agent(i,wealthinit.init_wealth(model), model))
    wealthinit.adapt_wealth(model)
    return