from mesa import Agent
import random
from agents.initialization import BeliefsInitializer

def random_markup():
    return random.randint(0,10)/100


class RandomMarkupAgent(Agent):
    """ An agent with a random markup added to its belief to determine the bidding/asking prices."""
    def __init__(agent, unique_id, wealth, model):
        super().__init__(unique_id, model)
        agent.wealth = wealth
        agent.belief = BeliefsInitializer().init_belief(model)
        agent.contracts_alpha = 0
        agent.contracts_beta = 0

    # Determines the ask price for alpha contracts
    def ask_price_alpha(self, model):
        return self.belief + random_markup()

    # Determines the ask price for beta contracts
    def ask_price_beta(self, model):
        return 1 - self.belief + random_markup()

    # Determines the bid price for alpha contracts
    def bid_price_alpha(self, model):
        return self.belief - random_markup()
    
    # Determines the bid price for beta contracts
    def bid_price_beta(self, model):
        return 1 - self.belief - random_markup()