# from agents.agents import init_belief
from mesa import Agent
import random
from model.model_statistics import *
from agents.initialization import BeliefsInitializer

class StrategicAgent(Agent):
    """ An agent which determines the supply and demand within the market and bases the bids and asks on that price.
    The better the situation, the further away from its belief. (the more profit)"""
    def __init__(agent, unique_id, wealth, model):
        super().__init__(unique_id, model)
        agent.wealth = wealth
        agent.belief = BeliefsInitializer().init_belief(model)
        agent.contracts_alpha = 0
        agent.contracts_beta = 0

    def ask_price_alpha(self, model):
        price = get_highest_ask(model.ask_queue_alpha).price  # Always place asks that are placed in the queue
        # The weight calculated using the supply and demand
        weight = (len(model.bid_queue_alpha)+1)/(len(model.bid_queue_alpha) + len(model.ask_queue_alpha)+1)
        return price - (price - self.belief - 0.01) * weight
            
    def ask_price_beta(self, model):
        price = get_highest_ask(model.ask_queue_beta).price  # Always place asks that are placed in the queue
        # The weight calculated using the supply and demand
        weight = (len(model.bid_queue_beta)+1)/(len(model.bid_queue_beta) + len(model.ask_queue_beta)+1)
        return price - (price - (1-self.belief) - 0.01) * weight

    def bid_price_alpha(self, model):
        price = get_lowest_bid(model.bid_queue_alpha).price # Always place bids that are placed in the queue
        # The weight calculated using the supply and demand
        weight = (len(model.ask_queue_alpha)+1)/(len(model.ask_queue_alpha) + len(model.bid_queue_alpha)+1)
        return price + (self.belief - 0.01 - price) * weight

    def bid_price_beta(self, model):
        price = get_lowest_bid(model.bid_queue_beta).price  # Always place bids that are placed in the queue
        # The weight calculated using the supply and demand
        weight = (len(model.ask_queue_beta)+1)/(len(model.ask_queue_beta) + len(model.bid_queue_beta)+1)
        return price + ((1-self.belief) - 0.01 - price) * weight