from mesa.batchrunner import BatchRunner, BatchRunnerMP
from model.model import MarketModel
import pandas as pd
import time
import csv

def createFirstRow():
    row = []
    row.append("num_adaptable")
    row.append("num_fixedmarkup")
    row.append("num_randommarkup")
    row.append("num_strategic")
    row.append("price_alpha")
    row.append("price_beta")
    row.append("average_belief")
    row.append("median_belief")
    row.append("wealth_distribution")
    row.append("belief_distribution")
    row.append("alpha_shape")
    row.append("mean_belief")
    row.append("sigma_belief")
    return row


# Params that do not change through the batch
fixed_params = {
                "total_wealth": 5000,
}

# Params that are changed for each run
variable_params = { 
                    "num_adaptable": [0,200],
                    "num_fixedmarkup": [0,200],
                    "num_randommarkup": [0,200],
                    "num_strategic": [0,200],
                    "wealth_distribution": ["Equal", "Random", "Pareto"],
                    "belief_distribution": ["Normal", "Random"],
                    "alpha_shape":[1.16, 1.42, 2.26],
                    "mean_belief": [0.25, 0.5, 0.75],
                    "sigma_belief": [0.125, 0.25]
                    }

if __name__ == '__main__':
    br = BatchRunnerMP(MarketModel, # Uses multithread processing
                            fixed_parameters=fixed_params,
                            variable_parameters=variable_params,
                            iterations=1, # Number of iterations per single configuration
                            max_steps=1000,
                            model_reporters={
                            	"Data Collector": lambda m: m.datacollector
                			})

if __name__ == '__main__':
    name = "./Data/{0}.csv".format(int(time.time())) # Gets the time and uses this as the name for the csv
    fixed_params['csv_name'] = name 
    createFirstRow()
    with open(name , 'w') as csvfile:
        writer = csv.writer(csvfile)
        writer.writerow(createFirstRow()) # Adds the headers for the csv
    br.run_all()