from mesa.visualization.ModularVisualization import ModularServer
from mesa.visualization.modules import *
from mesa.visualization.UserParam import UserSettableParameter
from model.model import MarketModel
from model.model_statistics import *

chart_element = ChartModule([{"Label": "Price_Alpha", "Color": "#3349FF"},
                            {"Label": "Price_Beta", "Color": "#FF0000"}])

chart_element1 = ChartModule([{"Label": "BidPrice_Alpha", "Color": "#3349FF"},
                            {"Label": "BidPrice_Beta", "Color": "#FF0000"}])

chart_element2 = ChartModule([{"Label": "LastPrice_Alpha", "Color": "#3349FF"},
                            {"Label": "LastPrice_Beta", "Color": "#FF0000"}])

# Some statistics about the beliefs and the number of trades
class StatisticsBeliefs(TextElement):
    def render(self, model):
        beliefs_mean = str(get_average_belief(model))
        beliefs_median = str(get_median_belief(model))
        beliefs_sd = str(sd_beliefs(model))
        alpha_trades = get_alpha_trades(model)
        beta_trades = get_beta_trades(model)
        return "Average belief: {} <br> Median belief: {} <br> Std beliefs: {} <br> Alpha trades: {} <br> Beta trades: {}".format(beliefs_mean, beliefs_median, beliefs_sd, alpha_trades, beta_trades)

# Some statistics about the queues
class Queues(TextElement):
    def render(self, model):
        queue1 = length_ask_queue_alpha(model)
        queue2 = length_bid_queue_alpha(model)
        queue3 = length_ask_queue_beta(model)
        queue4 = length_bid_queue_beta(model)
        units = get_units(model)
        return "units: {} <br> length AskQueueAlpha: {} <br> length BidQueueAlpha: {} <br> length AskQueueBeta: {} <br> length BidQueueBeta: {}".format(units , queue1, queue2,queue3,queue4)

# The input paramaters
model_params = {"num_adaptable": UserSettableParameter('slider', "Number of Adaptable Agents", 50, 0, 200, 10, description="Number of adaptable traders"),
                "num_fixedmarkup": UserSettableParameter('slider', "Number of FixedMarkup Agents", 50, 0, 200, 10, description="Number of fixed markup traders"),
                "num_randommarkup": UserSettableParameter('slider', "Number of RandomMarkup Agents", 50, 0, 200, 10, description="Number of random markup traders"),
                "num_strategic": UserSettableParameter('slider', "Number of Strategic Agents", 50, 0, 200, 10, description="Number of strategic traders"),
                "wealth_distribution": UserSettableParameter("choice", "Wealth Distribution", value="Equal",
                                           choices=["Random", "Equal", "Pareto"]),
                "total_wealth": UserSettableParameter('slider', "TotalWealth", 5000, 0, 7500, 100, description="Total wealth in the market"),
                "alpha_shape": UserSettableParameter('slider', "Alpha variable for shape pareto distribution", 1.16, 1.0, 2.0, 0.01, description="Wealth"),
                "belief_distribution": UserSettableParameter("choice", "belief Distribution", value="Random",
                                           choices=["Normal", "Random"]),
                "mean_belief": UserSettableParameter('slider', "meanbelief", 0.500, 0, 1, 0.25, description="Mean of belief (normal distribution)"),
                "sigma_belief": UserSettableParameter('slider', "sigmabelief", 0.125, 0, 1, 0.005, description="Standard deviation of belief (normal distribution)"),
                }

server = ModularServer(MarketModel, [StatisticsBeliefs(), Queues(), chart_element, chart_element1, chart_element2], "Prediction Market Model", model_params=model_params)
