from order.order import Order
import numpy as np

# This file contains all functions that are necessary for the DataCollector and other statistics 
# about the queues and agents. These functions are getters used in other functions
# They do not cause any action within the model.

def get_lowest_ask(queue):
    if(len(queue)>0):
        return queue[0]
    else:
        return Order(1, 0)

def get_highest_ask(queue):
    if(len(queue)>0):
        return queue[-1]
    else:
        return Order(0, 0)

def get_lowest_bid(queue):
    if(len(queue)>0):
        return queue[0]
    else:
        return Order(1, 0)

def get_highest_bid(queue):
    if(len(queue)>0):
        return queue[-1]
    else:
        return Order(0, 0)

def get_average_belief(model):
	try: model.average_belief
	except NameError: model.average_belief = None
	if model.average_belief is not None:
		return model.average_belief
	else:
		return 0

def get_median_belief(model): 
	try: model.median_belief
	except NameError: model.median_belief = None
	if model.median_belief is not None:
		return model.median_belief
	else:
		return 0

def get_alpha_trades(model):
	return len(model.trades_alpha)

def get_beta_trades(model):
	return len(model.trades_beta)

def length_ask_queue_alpha(model):
	return len(model.ask_queue_alpha)

def length_ask_queue_beta(model):
	return len(model.ask_queue_beta)

def length_bid_queue_alpha(model):
	return len(model.bid_queue_alpha)

def length_bid_queue_beta(model):
	return len(model.bid_queue_beta)

def track_run(model):
    return model.uid

def get_adaptable(model):
	return model.num_adaptable

def get_fixed_markup(model):
	return model.num_fixed_markup

def get_random_markup(model):
	return model.num_random_markup

def get_strategic(model):
	return model.num_strategic

def get_wealth_distribution(model):
	return model.wealth_distribution

def get_belief_distribution(model):
	return model.belief_distribution

def get_total_wealth(model):
	return model.total_wealth

def get_alpha_shape(model):
	return model.alpha_shape

def get_mean_belief(model):
	return model.mean_belief

def get_sigma_belief(model):
	return model.sigma_belief

def get_units(model):
	return model.unit_portfolios

def get_beliefs(model):
	beliefs = []
	for agent in model.schedule.agents:
		beliefs.append(agent.belief)
	return beliefs

def sd_beliefs(model):
    beliefs = []
    for agent in model.schedule.agents:
        beliefs.append(agent.belief)
    return np.std(beliefs)

def lowest_ask_alpha_price(model):
	if(len(model.ask_queue_alpha)>0):
		return model.ask_queue_alpha[0].price
	else:
		return 1

def highest_bid_alpha_price(model):
	if(len(model.bid_queue_alpha)>0):
		return model.bid_queue_alpha[-1].price
	else:
		return 0

def lowest_ask_beta_price(model):
    if(len(model.ask_queue_beta)>0):
    	return model.ask_queue_beta[0].price
    else:
    	return 1

def highest_bid_beta_price(model):
	if(len(model.bid_queue_beta)>0):
		return model.bid_queue_beta[-1].price
	else:
		return 0

def last_trade_alpha(model):
	length = len(model.trades_alpha)
	if(length > 0):
		return model.trades_alpha[length-1]
	else:
		return 0

def last_trade_beta(model):
	length = len(model.trades_beta)
	if(length > 0):
		return model.trades_beta[length-1]
	else:
		return 0